#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <string>

#include "sld.hh"

static Options get_default_options();

/*!
** Program entry point
**
** @param argc arg count
** @param argv arg vector
**
** @return program err code
*/
int main(int argc, char *argv[])
{
  SLDaemon &daemon = SLDaemon::Instance();
  Options *lineopt = new Options();

  try {
    // Set default options
    daemon.options.merge(get_default_options());

    // Parse and merge line options
    lineopt->getoptions(argc, argv);
    daemon.options.merge(*lineopt);

    // Get and apply options if conffile is specified
    if (!euidaccess(daemon.options.conffile.c_str(), R_OK)) {
      daemon.options.getoptions(daemon.options.conffile);

      //apply arg line options
      daemon.options.merge(*lineopt);
    }
  }
  catch (const Error &e) {
    e.print();
    return e.code();
  }

  delete lineopt;

  // register sigchild handler
  if (signal(SIGCHLD, sigchild) == SIG_ERR) {
    perror("sld: error");
    return ERR_UNKNOWN;
  }

  if (daemon.options.daemon == 1)
    ::daemon(0, 1);

  // run daemon
  try { daemon.run(); }
  catch (const Error &e) {
    std::cerr <<"sld: error: " << e.message() << "." << std::endl;
    return e.code();
  }

  return ERR_UNKNOWN;
}

/*!
** Set default options
**
** @param opt option struc to set
**
*/
static Options get_default_options() {
  char buf[128];
  Options options;

  if (gethostname(buf, 128) == 0)
    options.login = buf;
  options.port = 18136;
  options.conffile = "/etc/sldrc";
  options.retrydelay = 60;
  options.daemon = 0;
  return options;
}

/*!
** Retrieve complete path of this executable
**
** @return program path
*/
string getbinpath() {
  int pid = getpid();
  char buf[PATH_MAX];
  char buf2[PATH_MAX] = "";

  snprintf(buf, PATH_MAX, "/proc/%d/exe", pid);
  if (readlink(buf, buf2, PATH_MAX) == 0) {
    perror("sld: getbinpath: readlink: ");
    return (string) "";
  }
  return (string) buf2;
}

/*!
** copy src to dst
**
** @param src copy file src
** @param dst copy file dst
**
** @return true if all is copied
*/
bool cp(const char *src, const char *dst) {
  assert(src);
  assert(dst);

  //open source file
  FILE *fsrc = fopen(src, "r");
  if (!fsrc)
    return false;

  //open destination file
  FILE *fdst = fopen(dst, "w");
  if (!fdst) {
    fclose(fsrc);
    return false;
  }

  //do copy
  char buf[512];
  size_t nread;
  size_t nwrite;
  while ((nread = fread(buf, 1, 512, fsrc)) > 0) {
    nwrite = fwrite(buf, 1, nread, fdst);
    if (nwrite != nread) {
      fclose(fsrc);
      fclose(fdst);
      return false;
    }
  }
  fclose(fsrc);
  fclose(fdst);
  return true;
}
